@@ -1,4 +1,4 @@
-//@line 38 "/home/kojima/Firefox/build/browser/components/feeds/src/FeedConverter.js"
+//@line 39 "/mnt/Firefox/build/browser/components/feeds/src/FeedConverter.js"
 
 const Cc = Components.classes;
 const Ci = Components.interfaces;
@@ -19,11 +19,10 @@ const FPH_CLASSNAME = "Feed Protocol Handler";
 const PCPH_CONTRACTID = "@mozilla.org/network/protocol;1?name=pcast";
 const PCPH_CLASSID = Components.ID("{1c31ed79-accd-4b94-b517-06e0c81999d5}");
 const PCPH_CLASSNAME = "Podcast Protocol Handler";
-const FHS_CONTRACTID = "@mozilla.org/browser/feeds/handler-service;1";
-const FHS_CLASSID = Components.ID("{792a7e82-06a0-437c-af63-b2d12e808acc}");
-const FHS_CLASSNAME = "Feed Handler Service";
 
 const TYPE_MAYBE_FEED = "application/vnd.mozilla.maybe.feed";
+const TYPE_MAYBE_VIDEO_FEED = "application/vnd.mozilla.maybe.video.feed";
+const TYPE_MAYBE_AUDIO_FEED = "application/vnd.mozilla.maybe.audio.feed";
 const TYPE_ANY = "*/*";
 
 const FEEDHANDLER_URI = "about:feeds";
@@ -33,16 +32,66 @@ const PREF_SELECTED_WEB = "browser.feeds.handlers.webservice";
 const PREF_SELECTED_ACTION = "browser.feeds.handler";
 const PREF_SELECTED_READER = "browser.feeds.handler.default";
 
-function safeGetBoolPref(pref, defaultValue) {
-  var prefs =   
-      Cc["@mozilla.org/preferences-service;1"].
-      getService(Ci.nsIPrefBranch);
-  try {
-    return prefs.getBoolPref(pref);
+const PREF_VIDEO_SELECTED_APP = "browser.videoFeeds.handlers.application";
+const PREF_VIDEO_SELECTED_WEB = "browser.videoFeeds.handlers.webservice";
+const PREF_VIDEO_SELECTED_ACTION = "browser.videoFeeds.handler";
+const PREF_VIDEO_SELECTED_READER = "browser.videoFeeds.handler.default";
+
+const PREF_AUDIO_SELECTED_APP = "browser.audioFeeds.handlers.application";
+const PREF_AUDIO_SELECTED_WEB = "browser.audioFeeds.handlers.webservice";
+const PREF_AUDIO_SELECTED_ACTION = "browser.audioFeeds.handler";
+const PREF_AUDIO_SELECTED_READER = "browser.audioFeeds.handler.default";
+
+function getPrefAppForType(t) {
+  switch (t) {
+    case Ci.nsIFeed.TYPE_VIDEO:
+      return PREF_VIDEO_SELECTED_APP;
+
+    case Ci.nsIFeed.TYPE_AUDIO:
+      return PREF_AUDIO_SELECTED_APP;
+
+    default:
+      return PREF_SELECTED_APP;
   }
-  catch (e) {
+}
+
+function getPrefWebForType(t) {
+  switch (t) {
+    case Ci.nsIFeed.TYPE_VIDEO:
+      return PREF_VIDEO_SELECTED_WEB;
+
+    case Ci.nsIFeed.TYPE_AUDIO:
+      return PREF_AUDIO_SELECTED_WEB;
+
+    default:
+      return PREF_SELECTED_WEB;
+  }
+}
+
+function getPrefActionForType(t) {
+  switch (t) {
+    case Ci.nsIFeed.TYPE_VIDEO:
+      return PREF_VIDEO_SELECTED_ACTION;
+
+    case Ci.nsIFeed.TYPE_AUDIO:
+      return PREF_AUDIO_SELECTED_ACTION;
+
+    default:
+      return PREF_SELECTED_ACTION;
+  }
+}
+
+function getPrefReaderForType(t) {
+  switch (t) {
+    case Ci.nsIFeed.TYPE_VIDEO:
+      return PREF_VIDEO_SELECTED_READER;
+
+    case Ci.nsIFeed.TYPE_AUDIO:
+      return PREF_AUDIO_SELECTED_READER;
+
+    default:
+      return PREF_SELECTED_READER;
   }
-  return defaultValue;
 }
 
 function safeGetCharPref(pref, defaultValue) {
@@ -81,7 +130,9 @@ FeedConverter.prototype = {
    */
   canConvert: function FC_canConvert(sourceType, destinationType) {
     // We only support one conversion.
-    return destinationType == TYPE_ANY && sourceType == TYPE_MAYBE_FEED;
+    return destinationType == TYPE_ANY && ((sourceType == TYPE_MAYBE_FEED) ||
+                                           (sourceType == TYPE_MAYBE_VIDEO) ||
+                                           (sourceType == TYPE_MAYBE_AUDIO));
   },
   
   /**
@@ -104,15 +155,16 @@ FeedConverter.prototype = {
    * Whether or not the preview page is being forced.
    */
   _forcePreviewPage: false,
-
+  
   /** 
    * Release our references to various things once we're done using them.
    */
   _releaseHandles: function FC__releaseHandles() {
     this._listener = null;
     this._request = null;
+    this._processor = null;
   },
-
+  
   /**
    * See nsIFeedResultListener.idl
    */
@@ -152,18 +204,23 @@ FeedConverter.prototype = {
           Cc["@mozilla.org/browser/feeds/result-service;1"].
           getService(Ci.nsIFeedResultService);
       if (!this._forcePreviewPage && result.doc) {
-        var handler = safeGetCharPref(PREF_SELECTED_ACTION, "ask");
+        var feed = result.doc.QueryInterface(Ci.nsIFeed);
+        var handler = safeGetCharPref(getPrefActionForType(feed.type), "ask");
+
         if (handler != "ask") {
           if (handler == "reader")
-            handler = safeGetCharPref(PREF_SELECTED_READER, "bookmarks");
+            handler = safeGetCharPref(getPrefReaderForType(feed.type), "bookmarks");
           switch (handler) {
             case "web":
               var wccr = 
                   Cc["@mozilla.org/embeddor.implemented/web-content-handler-registrar;1"].
                   getService(Ci.nsIWebContentConverterService);
-              var feed = result.doc.QueryInterface(Ci.nsIFeed);
-              if (feed.type == Ci.nsIFeed.TYPE_FEED &&
-                  wccr.getAutoHandler(TYPE_MAYBE_FEED)) {
+              if ((feed.type == Ci.nsIFeed.TYPE_FEED &&
+                   wccr.getAutoHandler(TYPE_MAYBE_FEED)) ||
+                  (feed.type == Ci.nsIFeed.TYPE_VIDEO &&
+                   wccr.getAutoHandler(TYPE_MAYBE_VIDEO_FEED)) ||
+                  (feed.type == Ci.nsIFeed.TYPE_AUDIO &&
+                   wccr.getAutoHandler(TYPE_MAYBE_AUDIO_FEED))) {
                 wccr.loadPreferredHandler(this._request);
                 return;
               }
@@ -175,34 +232,34 @@ FeedConverter.prototype = {
             case "bookmarks":
             case "client":
               try {
-                var feed = result.doc.QueryInterface(Ci.nsIFeed);
                 var title = feed.title ? feed.title.plainText() : "";
                 var desc = feed.subtitle ? feed.subtitle.plainText() : "";
-                feedService.addToClientReader(result.uri.spec, title, desc);
+                feedService.addToClientReader(result.uri.spec, title, desc, feed.type);
                 return;
-              }
-              catch(ex) { /* fallback to preview mode */ }
+              } catch(ex) { /* fallback to preview mode */ }
           }
         }
       }
-
+          
       var ios = 
-           Cc["@mozilla.org/network/io-service;1"].
-           getService(Ci.nsIIOService);
+          Cc["@mozilla.org/network/io-service;1"].
+          getService(Ci.nsIIOService);
       var chromeChannel;
 
       // show the feed page if it wasn't sniffed and we have a document,
       // or we have a document, title, and link or id
       if (result.doc && (!this._sniffed ||
           (result.doc.title && (result.doc.link || result.doc.id)))) {
+
         // If there was no automatic handler, or this was a podcast,
         // photostream or some other kind of application, we must always
-        // show the preview page...
-
-        // Store the result in the result service so that the display page can 
-        // access it.
-        feedService.addFeedResult(result);
+        // show the preview page.
         
+        // Store the result in the result service so that the display
+        // page can access it.
+
+        feedService.addFeedResult(result);
+
         // Now load the actual XUL document.
         var chromeURI = ios.newURI(FEEDHANDLER_URI, null, null);
         chromeChannel = ios.newChannelFromURI(chromeURI, null);
@@ -224,8 +281,9 @@ FeedConverter.prototype = {
    */
   onDataAvailable: function FC_onDataAvailable(request, context, inputStream, 
                                                sourceOffset, count) {
-    this._processor.onDataAvailable(request, context, inputStream,
-                                    sourceOffset, count);
+    if (this._processor)
+      this._processor.onDataAvailable(request, context, inputStream,
+                                      sourceOffset, count);
   },
   
   /**
@@ -268,7 +326,8 @@ FeedConverter.prototype = {
    * See nsIRequestObserver.idl
    */
   onStopRequest: function FC_onStopReqeust(request, context, status) {
-    this._processor.onStopRequest(request, context, status);
+    if (this._processor)
+      this._processor.onStopRequest(request, context, status);
   },
   
   /**
@@ -296,7 +355,7 @@ var FeedConverterFactory = {
     if (iid.equals(Ci.nsIFactory) ||
         iid.equals(Ci.nsISupports))
       return this;
-    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
+    throw Cr.NS_ERROR_NO_INTERFACE;
   },
 };
 
@@ -313,31 +372,60 @@ var FeedResultService = {
   _results: { },
   
   /**
-   * See nsIFeedService.idl
+   * See nsIFeedResultService.idl
    */
   forcePreviewPage: false,
   
   /**
-   * See nsIFeedService.idl
+   * See nsIFeedResultService.idl
    */
-  addToClientReader: function FRS_addToClientReader(spec, title, subtitle) {
+  addToClientReader: function FRS_addToClientReader(spec, title, subtitle, feedType) {
     var prefs =   
         Cc["@mozilla.org/preferences-service;1"].
         getService(Ci.nsIPrefBranch);
 
-    var handler = safeGetCharPref(PREF_SELECTED_ACTION, "bookmarks");
-    if (handler == "ask" || handler == "reader")                                
-      handler = safeGetCharPref(PREF_SELECTED_READER, "bookmarks");             
+    var handler = safeGetCharPref(getPrefActionForType(feedType), "bookmarks");
+    if (handler == "ask" || handler == "reader")
+      handler = safeGetCharPref(getPrefReaderForType(feedType), "bookmarks");
 
     switch (handler) {
     case "client":
-      var clientApp = 
-        prefs.getComplexValue(PREF_SELECTED_APP, Ci.nsILocalFile);
-//@line 393 "/home/kojima/Firefox/build/browser/components/feeds/src/FeedConverter.js"
-      var ss = 
-          Cc["@mozilla.org/browser/shell-service;1"].
-          getService(Ci.nsIShellService_MOZILLA_1_8_BRANCH);
-      ss.openApplicationWithURI(clientApp, spec);
+      var clientApp = prefs.getComplexValue(getPrefAppForType(feedType), Ci.nsILocalFile);
+
+      // For the benefit of applications that might know how to deal with more
+      // URLs than just feeds, send feed: URLs in the following format:
+      //
+      // http urls: replace scheme with feed, e.g.
+      // http://foo.com/index.rdf -> feed://foo.com/index.rdf
+      // other urls: prepend feed: scheme, e.g.
+      // https://foo.com/index.rdf -> feed:https://foo.com/index.rdf
+      var ios = 
+          Cc["@mozilla.org/network/io-service;1"].
+          getService(Ci.nsIIOService);
+      var feedURI = ios.newURI(spec, null, null);
+      if (feedURI.schemeIs("http")) {
+        feedURI.scheme = "feed";
+        spec = feedURI.spec;
+      }
+      else
+        spec = "feed:" + spec;
+
+      // Retrieving the shell service might fail on some systems, most
+      // notably systems where GNOME is not installed.
+      try {
+        var ss =
+            Cc["@mozilla.org/browser/shell-service;1"].
+            getService(Ci.nsIShellService);
+        ss.openApplicationWithURI(clientApp, spec);
+      } catch(e) {
+        // If we couldn't use the shell service, fallback to using a
+        // nsIProcess instance
+        var p =
+            Cc["@mozilla.org/process/util;1"].
+            createInstance(Ci.nsIProcess);
+        p.init(clientApp);
+        p.run(false, [spec], 1);
+      }
       break;
 
     default:
@@ -349,15 +437,13 @@ var FeedResultService = {
           Cc["@mozilla.org/appshell/window-mediator;1"].
           getService(Ci.nsIWindowMediator);
       var topWindow = wm.getMostRecentWindow("navigator:browser");
-//@line 411 "/home/kojima/Firefox/build/browser/components/feeds/src/FeedConverter.js"
-      topWindow.FeedHandler.addLiveBookmark(spec, title, subtitle);
-//@line 413 "/home/kojima/Firefox/build/browser/components/feeds/src/FeedConverter.js"
+      topWindow.PlacesCommandHook.addLiveBookmark(spec, title, subtitle);
       break;
     }
   },
   
   /**
-   * See nsIFeedService.idl
+   * See nsIFeedResultService.idl
    */
   addFeedResult: function FRS_addFeedResult(feedResult) {
     NS_ASSERT(feedResult.uri != null, "null URI!");
@@ -369,7 +455,7 @@ var FeedResultService = {
   },
   
   /**
-   * See nsIFeedService.idl
+   * See nsIFeedResultService.idl
    */
   getFeedResult: function RFS_getFeedResult(uri) {
     NS_ASSERT(uri != null, "null URI!");
@@ -382,7 +468,7 @@ var FeedResultService = {
   },
   
   /**
-   * See nsIFeedService.idl
+   * See nsIFeedResultService.idl
    */
   removeFeedResult: function FRS_removeFeedResult(uri) {
     NS_ASSERT(uri != null, "null URI!");
@@ -421,9 +507,8 @@ var FeedResultService = {
 };
 
 /**
- * A protocol handler that converts the URIs of Apple's various bogo protocol
- * schemes into http, as they should be. Mostly, this object just forwards 
- * things through to the HTTP protocol handler.
+ * A protocol handler that attempts to deal with the variant forms of feed:
+ * URIs that are actually either http or https.
  */
 function FeedProtocolHandler(scheme) {
   this._scheme = scheme;
@@ -451,6 +536,23 @@ FeedProtocolHandler.prototype = {
   },
   
   newURI: function FPH_newURI(spec, originalCharset, baseURI) {
+    // See bug 408599 - feed URIs can be either standard URLs of the form
+    // feed://example.com, in which case the real protocol is http, or nested
+    // URIs of the form feed:realscheme:. When realscheme is either http or
+    // https, we deal with the way that creates a standard URL with the
+    // realscheme as the host by unmangling in newChannel; for others, we fail
+    // rather than let it wind up loading something like www.realscheme.com//foo
+
+    const feedSlashes = "feed://";
+    const feedHttpSlashes = "feed:http://";
+    const feedHttpsSlashes = "feed:https://";
+    const NS_ERROR_MALFORMED_URI = 0x804B000A;
+
+    if (spec.substr(0, feedSlashes.length) != feedSlashes &&
+        spec.substr(0, feedHttpSlashes.length) != feedHttpSlashes &&
+        spec.substr(0, feedHttpsSlashes.length) != feedHttpsSlashes)
+      throw NS_ERROR_MALFORMED_URI;
+
     var uri = 
         Cc["@mozilla.org/network/standard-url;1"].
         createInstance(Ci.nsIStandardURL);
@@ -459,26 +561,28 @@ FeedProtocolHandler.prototype = {
     return uri;
   },
   
-  newChannel: function FPH_newChannel(uri) {
+  newChannel: function FPH_newChannel(aUri) {
     var ios = 
         Cc["@mozilla.org/network/io-service;1"].
         getService(Ci.nsIIOService);
     // feed: URIs either start feed://, in which case the real scheme is http:
     // or feed:http(s)://, (which by now we've changed to feed://realscheme//)
+    var feedSpec = aUri.spec;
     const httpsChunk = "feed://https//";
     const httpChunk = "feed://http//";
-    if (uri.spec.substr(0, httpsChunk.length) == httpsChunk)
-      uri.spec = "https://" + uri.spec.substr(httpsChunk.length);
-    else if (uri.spec.substr(0, httpChunk.length) == httpChunk)
-      uri.spec = "http://" + uri.spec.substr(httpChunk.length);
+    if (feedSpec.substr(0, httpsChunk.length) == httpsChunk)
+      feedSpec = "https://" + feedSpec.substr(httpsChunk.length);
+    else if (feedSpec.substr(0, httpChunk.length) == httpChunk)
+      feedSpec = "http://" + feedSpec.substr(httpChunk.length);
     else
-      uri.scheme = "http";
+      feedSpec = feedSpec.replace(/^feed/, "http");
 
+    var uri = ios.newURI(feedSpec, aUri.originCharset, null);
     var channel =
       ios.newChannelFromURI(uri, null).QueryInterface(Ci.nsIHttpChannel);
     // Set this so we know this is supposed to be a feed
     channel.setRequestHeader("X-Moz-Is-Feed", "1", false);
-    channel.originalURI = uri;
+    channel.originalURI = aUri;
     return channel;
   },
   
@@ -531,7 +635,17 @@ var Module = {
         converterPrefix + TYPE_MAYBE_FEED + "&to=" + TYPE_ANY;
     cr.registerFactoryLocation(FC_CLASSID, FC_CLASSNAME, converterContractID,
                                file, location, type);
-  },
+
+    converterContractID = 
+        converterPrefix + TYPE_MAYBE_VIDEO_FEED + "&to=" + TYPE_ANY;
+    cr.registerFactoryLocation(FC_CLASSID, FC_CLASSNAME, converterContractID,
+                               file, location, type);
+
+    converterContractID = 
+        converterPrefix + TYPE_MAYBE_AUDIO_FEED + "&to=" + TYPE_ANY;
+    cr.registerFactoryLocation(FC_CLASSID, FC_CLASSNAME, converterContractID,
+                               file, location, type);
+    },
   
   unregisterSelf: function M_unregisterSelf(cm, location, type) {
     var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);
@@ -548,7 +662,9 @@ function NSGetModule(cm, file) {
   return Module;
 }
 
-//@line 44 "/home/kojima/Firefox/build/browser/components/feeds/src/../../../../toolkit/content/debug.js"
+//@line 44 "/mnt/Firefox/build/toolkit/content/debug.js"
+
+var EXPORTED_SYMBOLS = ["NS_ASSERT"];
 
 var gTraceOnAssert = true;
 
@@ -573,14 +689,62 @@ function NS_ASSERT(condition, message) {
   if (condition)
     return;
 
+  var releaseBuild = true;
+  var defB = Components.classes["@mozilla.org/preferences-service;1"]
+                       .getService(Components.interfaces.nsIPrefService)
+                       .getDefaultBranch(null);
+  try {
+    switch (defB.getCharPref("app.update.channel")) {
+      case "nightly":
+      case "beta":
+      case "default":
+        releaseBuild = false;
+    }
+  } catch(ex) {}
+
+  var caller = arguments.callee.caller;
   var assertionText = "ASSERT: " + message + "\n";
 
-//@line 72 "/home/kojima/Firefox/build/browser/components/feeds/src/../../../../toolkit/content/debug.js"
-  Components.util.reportError(assertionText);
-  return;
-//@line 108 "/home/kojima/Firefox/build/browser/components/feeds/src/../../../../toolkit/content/debug.js"
+  if (releaseBuild) {
+    // Just report the error to the console
+    Components.utils.reportError(assertionText);
+    return;
+  }
+
+  // Otherwise, dump to stdout and launch an assertion failure dialog
+  dump(assertionText);
+
+  var stackText = "";
+  if (gTraceOnAssert) {
+    stackText = "Stack Trace: \n";
+    var count = 0;
+    while (caller) {
+      stackText += count++ + ":" + caller.name + "(";
+      for (var i = 0; i < caller.arguments.length; ++i) {
+        var arg = caller.arguments[i];
+        stackText += arg;
+        if (i < caller.arguments.length - 1)
+          stackText += ",";
+      }
+      stackText += ")\n";
+      caller = caller.arguments.callee.caller;
+    }
+  }
+
+  var environment = Components.classes["@mozilla.org/process/environment;1"].
+                    getService(Components.interfaces.nsIEnvironment);
+  if (environment.exists("XUL_ASSERT_PROMPT") &&
+      !parseInt(environment.get("XUL_ASSERT_PROMPT")))
+    return;
+
+  var source = null;
+  if (this.window)
+    source = this.window;
+  var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
+           getService(Components.interfaces.nsIPromptService);
+  ps.alert(source, "Assertion Failed", assertionText + stackText);
 }
-//@line 37 "/home/kojima/Firefox/build/browser/components/feeds/src/GenericFactory.js"
+//@line 37 "/mnt/Firefox/build/browser/components/feeds/src/GenericFactory.js"
 
 /**
  * An object implementing nsIFactory that can construct other objects upon
